(function () {
    var factory = function (exports) {
        var $ = jQuery; // if using module loader(Require.js/Sea.js).

        exports.fn.cnbSearchReplaceDialog = function () {
            var classPrefix = this.classPrefix;
            var pluginName = "cnb-search-replace";
            var dialogHtml =
                '<div class="' +
                classPrefix +
                'form">' +
                "<label>" +
                "查找目标" +
                "</label>" +
                '<input type="text" data-search-target />' +
                "<br/>" +
                "<label>" +
                "替换" +
                "</label>" +
                '<input type="text" data-replace-content />' +
                "<br/>" +
                "</div>";
            var cm = this.cm;
            var editor = this.editor;
            var dialogName = classPrefix + pluginName;
            var dialog = editor.find("." + dialogName);
            var findState = {};
            const searchTextEl = () => {
                return document.querySelector(
                    "." + dialogName + " input[data-search-target]"
                );
            };

            const search = (text, start = undefined) => {
                findState.searchText = text;
                const cursor = cm.getSearchCursor(text, start);
                const positions = [];
                while (cursor.findNext()) {
                    positions.push({
                        anchor: cursor.from(),
                        head: cursor.to(),
                    });
                }
                findState.positions = positions;
                findState.posIndex = 0;
            };

            const getSearchText = () => {
                const el = searchTextEl();
                return el ? el.value || "" : "";
            };
            const getReplaceText = () => {
                const replaceTextEl = document.querySelector(
                    "." + dialogName + " input[data-replace-content]"
                );
                return replaceTextEl ? replaceTextEl.value || "" : "";
            };
            const findNext = () => {
                const searchText = getSearchText();
                if (findState.searchText !== searchText) {
                    search(searchText);
                    if (findState.positions.length > 0)
                        cm.setSelections([
                            findState.positions[findState.posIndex++],
                        ]);
                } else {
                    if (findState.posIndex < findState.positions.length) {
                        cm.setSelections([
                            findState.positions[findState.posIndex++],
                        ]);
                    } else {
                        findState = {};
                        findNext();
                    }
                }
            };
            const findAll = () => {
                const searchText = getSearchText();
                if (findState.searchText !== searchText) {
                    search(searchText);
                }
                cm.setSelections(findState.positions);
            };
            if (dialog.length > 0) {
                dialog.remove();
            }
            this.createDialog({
                name: dialogName,
                title: "查找/替换",
                width: 400,
                height: 211,
                content: dialogHtml,
                lockScreen: true,
                mask: true,
                buttons: {
                    search: [
                        "查找",
                        function () {
                            findNext();
                        },
                    ],
                    searchAll: [
                        "查找全部",
                        function () {
                            findAll();
                        },
                    ],
                    searchReplace: [
                        "替换",
                        function () {
                            let selections = cm.listSelections();
                            let selection =
                                selections && selections.length > 0
                                    ? selections[0]
                                    : {};
                            search(
                                getSearchText(),
                                findState.nextReplaceStartPosition ||
                                    selection.anchor
                            );
                            if (
                                findState &&
                                findState.positions &&
                                findState.positions.length > 0
                            ) {
                                cm.setSelections([findState.positions[0]]);
                                const replaceText = getReplaceText();
                                cm.replaceSelection(replaceText, "around");
                                // 计算下次替换时开始位置
                                selections = cm.listSelections();
                                let selection =
                                    selections && selections.length > 0
                                        ? selections[0]
                                        : undefined;
                                findState = {
                                    nextReplaceStartPosition: selection
                                        ? selection.head
                                        : undefined,
                                };
                            }
                        },
                    ],
                    searchReplaceAll: [
                        "替换全部",
                        function () {
                            search(getSearchText());
                            if (
                                findState &&
                                findState.positions &&
                                findState.positions.length > 0
                            ) {
                                cm.setSelections(findState.positions);
                                const replaceText = getReplaceText();
                                cm.replaceSelection(replaceText, "around");
                                findState = {};
                            }
                        },
                    ],
                },
            });
            setTimeout(() => searchTextEl().focus(), 100);
        };
    };

    // CommonJS/Node.js
    if (
        typeof require === "function" &&
        typeof require === "function" &&
        typeof exports === "object" &&
        typeof module === "object"
    ) {
        module.exports = factory;
    } else {
        factory(window.editormd);
    }
})();
